/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.mock;

import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.wsdl.BindingOperation;
import javax.wsdl.Part;
import javax.xml.namespace.QName;

import org.apache.xmlbeans.XmlObject;
import org.mortbay.jetty.Request;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.support.soap.SoapVersion;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlUtils;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestRunContext;
import com.eviware.soapui.model.mock.MockResult;
import com.eviware.soapui.model.mock.MockRunListener;
import com.eviware.soapui.model.mock.MockRunner;
import com.eviware.soapui.support.xml.XmlUtils;

/**
 * MockRunner that dispatches Http Requests to their designated WsdlMockOperation if possible
 * 
 * @author ole.matzura
 */

public class WsdlMockRunner implements MockRunner
{
	private WsdlMockService mockService;
	private List<WsdlMockResult> mockResults = Collections.synchronizedList( new LinkedList<WsdlMockResult>());
	private long maxResults = 100;
	private int removed = 0;
	private WsdlMockRunContext mockContext;

	public WsdlMockRunner( WsdlMockService mockService, WsdlTestRunContext context ) throws Exception
	{
		this.mockService = mockService;
		
		Set<WsdlInterface> interfaces = new HashSet<WsdlInterface>();
		
		for( int i = 0; i < mockService.getMockOperationCount(); i++ )
		{
			WsdlOperation operation = mockService.getMockOperationAt( i ).getOperation();
			if( operation != null )
				interfaces.add( operation.getInterface() );
		}
		
		for( WsdlInterface iface : interfaces )
			iface.getWsdlContext().loadIfNecessary( false );
		
		mockContext = new WsdlMockRunContext( mockService, context );
		
		SoapUI.getMockEngine().startMockService( this );
		
		MockRunListener[] mockRunListeners = mockService.getMockRunListeners();
		
		for( MockRunListener listener : mockRunListeners )
		{
			listener.onMockRunnerStart( this );
		}
	}

	public WsdlMockRunContext getMockContext()
	{
		return mockContext;
	}

	public synchronized void addMockResult( WsdlMockResult mockResult )
	{
		mockResults.add( mockResult );
		while( mockResults.size() > maxResults )
		{
			mockResults.remove( 0 );
			removed++;
		}
	}
	
	public void stop()
	{
		SoapUI.getMockEngine().stopMockService( this );
		
		MockRunListener[] mockRunListeners = mockService.getMockRunListeners();
		
		for( MockRunListener listener : mockRunListeners )
		{
			listener.onMockRunnerStop( this );
		}
	}

	public WsdlMockService getMockService()
	{
		return mockService;
	}
	
	public long getMaxResults()
	{
		return maxResults;
	}

	public synchronized void setMaxResults( long l )
	{
		this.maxResults = l;
		
		while( mockResults.size() > l )
		{
			mockResults.remove( 0 );
			removed++;
		}
	}

	@SuppressWarnings("unchecked")
	public WsdlMockResult dispatchRequest( HttpServletRequest request, HttpServletResponse response ) throws DispatchException
	{
		WsdlMockResult result = null;
		MockRunListener[] mockRunListeners = mockService.getMockRunListeners();
		
		try
		{
			for( MockRunListener listener : mockRunListeners )
			{
				listener.onMockRequest( this, request, response );
			}
			
			long timestamp = System.currentTimeMillis();
			String soapAction = request.getHeader( "SOAPAction" );
			
			WsdlMockRequest mockRequest = new WsdlMockRequest( request, response, mockContext );
			
			SoapVersion soapVersion = mockRequest.getSoapVersion();
			if( soapVersion == null )
				throw new DispatchException( "Unrecognized SOAP Version" );
			
			XmlObject contentElm = mockRequest.getContentElement(); 
			if( contentElm == null )
				throw new DispatchException( "Missing content element in body" );
			
			QName contentQName = XmlUtils.getQName( contentElm.getDomNode() );
			NodeList contentChildNodes = null;
			
			for( int c = 0; c < mockService.getMockOperationCount(); c++ )
			{
				WsdlMockOperation mockOperation = ( WsdlMockOperation ) mockService.getMockOperationAt( c );
				WsdlOperation wsdlOperation = ( WsdlOperation ) mockOperation.getOperation();
				if( wsdlOperation == null )
					continue;

				if( mockService.isRequireSoapVersion() && 
					 wsdlOperation.getInterface().getSoapVersion() != mockRequest.getSoapVersion() )
				{
					continue;
				}
				
				String action = wsdlOperation.getAction() == null ? "\"\"" : "\"" + wsdlOperation.getAction() + "\"";
				
				// matches soapAction?
				if( (soapAction == null && wsdlOperation.getAction() == null ) || 
					 (action.equals( soapAction ) || (wsdlOperation.getAction() != null && wsdlOperation.getAction().equals( soapAction ) )))
				{
					QName qname = wsdlOperation.getRequestBodyElementQName();
					
					if( !qname.equals( contentQName ))
						continue;
					
					long startTime = 0;
					
					// check content
					if( wsdlOperation.getStyle().equals( WsdlOperation.STYLE_DOCUMENT ))
					{
						// matches!
						startTime = System.nanoTime();
						result = mockOperation.dispatchRequest( mockRequest, response );
					}
					else if( wsdlOperation.getStyle().equals( WsdlOperation.STYLE_RPC ))
					{
						BindingOperation bindingOperation = wsdlOperation.getBindingOperation();
						List<Part> parts = bindingOperation.getOperation().getInput().getMessage().getOrderedParts( null );
						
						if( contentChildNodes == null )
							contentChildNodes = XmlUtils.getChildElements( ( Element ) contentElm.getDomNode() );
						
						int i = 0;
						
						if( parts.size() > 0 )
						{
							for( int x = 0; x < parts.size(); x++ )
							{
								if( WsdlUtils.isAttachmentInputPart( parts.get( x ), bindingOperation ))
								{
									parts.remove( x );
									x--;
								}
							}
							
							for( ; i < contentChildNodes.getLength() && !parts.isEmpty(); i++ )
							{
								Node item = contentChildNodes.item( i );
								if( item.getNodeType() != Node.ELEMENT_NODE )
									continue;
								
								int j=0; 
								while ((j<parts.size()) && (!item.getNodeName().equals( parts.get( j ).getName()))) 
								{ 
									j++; 
								} 
								
								if (j==parts.size()) 
									break; 
								 
								parts.remove( j ); 
							}
						}
						
						// match?
						if( i == contentChildNodes.getLength() && parts.isEmpty() )
						{
							startTime = System.nanoTime();
							result = mockOperation.dispatchRequest( mockRequest, response );
						}
					}
					
					if( startTime == 0 )
					{
						throw new DispatchException( "Failed to find matching operation for request" );
					}
					else
					{
						((Request)request).setHandled( true );
						result.setTimeTaken( ( System.nanoTime() - startTime ) / 1000000 );
						result.setTimestamp( timestamp );
						addMockResult( result );
						return result;
					}
				}
			}
			
			throw new DispatchException( "Missing operation for soapAction [" + soapAction + "] and body element [" + 
						contentQName + "] with SOAP Version [" + mockRequest.getSoapVersion() + "]" );
		}
		catch( Exception e )
		{
			if( e instanceof DispatchException )
				throw (DispatchException)e;
			
			throw new DispatchException( e );
		}
		finally
		{
			if( result != null )
			{
				for( MockRunListener listener : mockRunListeners )
				{
					listener.onMockResult( result );
				}
			}
		}
	}

	public MockResult getMockResultAt( int index )
	{
		return index <= removed ? null : mockResults.get( index-removed );
	}

	public int getMockResultCount()
	{
		return mockResults.size() + removed;
	}

	public synchronized void clearResults()
	{
		mockResults.clear();
	}
	
	public void release()
	{
		clearResults();
		mockService = null;
		mockContext.clear();
	}
}
