/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.apache.log4j.Priority;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.BaseHttpRequestTransport;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.support.StringUtils;
import com.eviware.soapui.support.xml.XmlUtils;

/**
 * RequestFilter that expands properties in request content
 * 
 * @author Ole.Matzura
 */

public class PropertyExpansionRequestFilter implements RequestFilter
{
	private static final char PROPERTY_SEPARATOR = '#';
	private final static Logger log = Logger.getLogger(PropertyExpansionRequestFilter.class);
	private static Properties globalProperties;
	
	public void filterRequest(SubmitContext context, WsdlRequest wsdlRequest)
	{
		String content = (String) context.getProperty( BaseHttpRequestTransport.REQUEST_CONTENT );
		if( content == null )
		{
			log.warn( "Missing request content in context, skipping property expansion" );
		}
		else
		{
			content = expandProperties(context, content);
			if( content != null )
				context.setProperty( BaseHttpRequestTransport.REQUEST_CONTENT, content );
		}
	}

	@SuppressWarnings("deprecation")
	public static String expandProperties(SubmitContext context, String content)
	{
		if( StringUtils.isNullOrEmpty( content ))
			return content;
		
		int ix = content.indexOf( "${" );
		if( ix == -1 )
			return content;
		
		StringBuffer buf = new StringBuffer();
		int lastIx = 0;
		while( ix != -1 )
		{
			if( ix > lastIx )
			   buf.append( content.substring( lastIx, ix ));
			
			int ix2 = content.indexOf( '}', ix+2 );
			if( ix2 == -1 )
				break;

			// check for nesting
			int ix3 = content.lastIndexOf( "${", ix2 );
			if( ix3 != ix )
			{
				//buf.append( content.substring( ix, ix3 ));
				content = content.substring( 0, ix3 ) + expandProperties( context, content.substring( ix3, ix2+1 )) + 
				          content.substring( ix2+1 );
				
				lastIx = ix;
				continue;
			}
			
			String propertyName = content.substring( ix+2, ix2 );
			
			// check for system-property reference
			Object property = propertyName.length() > 2 && propertyName.charAt( 0 ) == '#' && propertyName.charAt( 1 ) == '#' ? 
						getSystemProperty( propertyName.substring( 2 ) ) : context.getProperty( propertyName );
			
			if( property != null )
			{
				String value = property.toString();
				if( !content.equals( value ))
					value = expandProperties( context, value );
				
				buf.append( value );
			}
			else if( context instanceof TestRunContext )
			{
				int sepIx = propertyName.indexOf( PROPERTY_SEPARATOR );
				if( sepIx >= 0 )
				{
					String step = propertyName.substring( 0, sepIx );
					String name = propertyName.substring( sepIx+1 );
					
					sepIx = name.indexOf( PROPERTY_SEPARATOR );
					if( sepIx != -1 )
					{
						String xpath = name.substring( sepIx+1 );
						name = name.substring( 0, sepIx );
						
						if( step.length() == 0 )
							property = ((TestRunContext)context).getProperty( name);
						else
							property = ((TestRunContext)context).getProperty( step, name);
						
						if( property != null )
						{
							property = extractXPathPropertyValue( property, xpath );
						}
					}
					else
					{
						if( step.length() == 0 )
							property = ((TestRunContext)context).getProperty( name);
						else
							property = ((TestRunContext)context).getProperty( step, name);
					}
					
					if( property != null )
					{
						String value = property.toString();
						if( !content.equals( value ))
							value = expandProperties( context, value );
						
						buf.append( value );
					}
				}
			}
			else if( propertyName.charAt( 0 ) == PROPERTY_SEPARATOR  )
			{
				int sepIx = propertyName.indexOf( PROPERTY_SEPARATOR, 1 );
				if( sepIx > 0 )
				{
					String xpath = propertyName.substring( sepIx+1 );
					propertyName = propertyName.substring( 1, sepIx );
					
					property = context.getProperty( propertyName );
					if( property != null )
					{
						property = extractXPathPropertyValue( property, expandProperties( context, xpath ));
					}
				}
				else
				{
					property = context.getProperty( propertyName.substring( 1 ) );
				}
				
				if( property != null )
				{
					String value = property.toString();
					if( !content.equals( value ))
						value = expandProperties( context, value );
					
					buf.append( value );
				}
			}
				
			if( property == null )
			{
				if( log.isEnabledFor( Priority.WARN ))
					log.warn( "Missing property value for [" + propertyName + "]" );
				
			   buf.append( "${" ).append( propertyName ).append( '}' );
			}
			
			lastIx = ix2+1;
			ix = content.indexOf( "${", lastIx );
		}
		
		if( lastIx < content.length() )
			buf.append( content.substring( lastIx ));
		
		return buf.toString();
	}

	private static String getSystemProperty( String propertyName )
	{
		if( globalProperties == null )
		{
			globalProperties = new Properties();
		   String propFile = System.getProperty( "soapui.properties" );
		   File file = new File( propFile );
			if( file.exists() )
		   {
				try
				{
					globalProperties.load( new FileInputStream( file ) );
					log.info( "Loaded " + globalProperties.size() + " properties from [" + file.getAbsolutePath() + "]" );
				}
				catch( Exception e )
				{
					e.printStackTrace();
				}
		   }
		}
		
		return globalProperties.isEmpty() ? System.getProperty( propertyName ) :
			globalProperties.getProperty( propertyName, System.getProperty( propertyName ));
	}

	public static String extractXPathPropertyValue( Object property, String xpath )
	{
		try
		{
			XmlObject xmlObject = XmlObject.Factory.parse(  property.toString() );
			String ns = XmlUtils.declareXPathNamespaces( xmlObject );
			XmlObject[] paths = xmlObject.selectPath( ns + xpath );
			if( paths.length > 0 )
				return XmlUtils.getNodeValue( paths[0].getDomNode() );
		}
		catch( Exception e )
		{
			SoapUI.logError( e );
		}
		
		return null;
	}
}
