/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.ws.security.WSConstants;
import org.apache.ws.security.message.WSSecHeader;
import org.apache.ws.security.message.WSSecTimestamp;
import org.apache.ws.security.message.WSSecUsernameToken;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.BaseHttpRequestTransport;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.support.StringUtils;
import com.eviware.soapui.support.xml.XmlUtils;

/**
 * Modifies the request message to include WS-Securty Username and Timestamp tokens
 * 
 * @author Ole.Matzura
 */

public class WsSecurityAuthenticationRequestFilter implements RequestFilter
{
	private static DocumentBuilderFactory dbf;
	private static DocumentBuilder db;

	static
	{
		dbf = DocumentBuilderFactory.newInstance();
		dbf.setValidating(false);
	   dbf.setNamespaceAware(true);
	   
	   try
		{
			db = dbf.newDocumentBuilder();
		}
		catch (ParserConfigurationException e)
		{
			SoapUI.logError( e );
		}
	}

	public void filterRequest(SubmitContext context, WsdlRequest wsdlRequest)
	{
		String pwType = wsdlRequest.getWssPasswordType();
      String wsTimestamp = wsdlRequest.getWssTimeToLive();
      
		if (  (StringUtils.isNullOrEmpty( pwType ) || WsdlRequest.PW_TYPE_NONE.equals(pwType)) &&
      		(StringUtils.isNullOrEmpty( wsTimestamp )))
      		return;
      try 
      {
        	addWssHeaders( context, wsdlRequest.getUsername(), wsdlRequest.getPassword(), pwType, wsTimestamp );
      } 
		catch (Throwable e) 
		{
          SoapUI.logError( e );
      }
	}

	public static void addWssHeaders( SubmitContext context, String username, String password, String pwType, String wsTimestamp ) throws SAXException, IOException
	{
		String request = (String) context.getProperty( BaseHttpRequestTransport.REQUEST_CONTENT );
		Document doc = null;
		
		// this should be solved with pooling for performance-reasons..
		synchronized( db )
		{
			doc = db.parse(new InputSource( new StringReader( request )));	
		}
		
		if( StringUtils.hasContent( pwType ) && !pwType.equals(  WsdlRequest.PW_TYPE_NONE ) && 
			 StringUtils.hasContent( username ) && StringUtils.hasContent( password ))
			addWssUsernameToken( username, password, pwType, doc );

		if( StringUtils.hasContent( wsTimestamp ) )
			addWsTimestampToken( wsTimestamp, doc );

		StringWriter writer = new StringWriter();
		XmlUtils.serializePretty( doc, writer );
		context.setProperty( BaseHttpRequestTransport.REQUEST_CONTENT, writer.toString() );
	}

	private static void addWsTimestampToken( String ttl, Document doc )
	{
		WSSecTimestamp addTimestamp = new WSSecTimestamp();
		addTimestamp.setTimeToLive( Integer.parseInt( ttl ));
		WSSecHeader secHeader = new WSSecHeader();
		secHeader.insertSecurityHeader( doc );
		addTimestamp.build( doc, secHeader );
	}

	private static void addWssUsernameToken( String username, String password, String pwType, Document doc )
	{
		WSSecUsernameToken wsa = new WSSecUsernameToken();
		if (WsdlRequest.PW_TYPE_DIGEST.equals(pwType)) 
		{
		   wsa.setPasswordType(WSConstants.PASSWORD_DIGEST);
		} 
		else 
		{
		   wsa.setPasswordType(WSConstants.PASSWORD_TEXT);
		}
		
		wsa.setUserInfo(username, password ); 
		
		WSSecHeader secHeader = new WSSecHeader();
		secHeader.insertSecurityHeader( doc );
		wsa.build(doc, secHeader );
	}
}
