package com.eviware.soapui.support.components;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import com.eviware.soapui.support.UISupport;

public class JInspectorPanel extends JPanel implements PropertyChangeListener
{	
	private float defaultDividerLocation = 0.7F;
	
	private final JSplitPane mainSplit;
	private JPanel inspectorPanel;
	private int lastDividerLocation = 0;
	private JXToolBar inspectToolbar;
	private List<Inspector> inspectors = new ArrayList<Inspector>();
	private Map<Inspector,JToggleButton> inspectorButtons = new HashMap<Inspector,JToggleButton>();
	public Inspector currentInspector;

	private final int orientation;

	public JInspectorPanel( JComponent contentComponent )
	{
		this( contentComponent, JSplitPane.VERTICAL_SPLIT );
	}
	
	public JInspectorPanel(JComponent contentComponent, int orientation )
	{
		super( new BorderLayout() );
		this.orientation = orientation;

		inspectorPanel = new JPanel(new CardLayout());
		inspectorPanel.setVisible( false );
		
		mainSplit = new JSplitPane( orientation );
		mainSplit.setDividerSize( 6 );
		mainSplit.setBorder( null );
		mainSplit.setOneTouchExpandable( false );
		
		JXToolBar toolbar = createInspectButtons();
		if( orientation == JSplitPane.VERTICAL_SPLIT )
		{
			mainSplit.setTopComponent( contentComponent );
			mainSplit.setBottomComponent( inspectorPanel );
			mainSplit.setResizeWeight( 0.8 );
			toolbar.setBorder( BorderFactory.createEmptyBorder( 1, 2, 3, 2 ));
			add( toolbar, BorderLayout.SOUTH  );
		}
		else
		{
			mainSplit.setRightComponent( contentComponent );
			
			JPanel p = new JPanel( new BorderLayout() );
			p.add( toolbar );
			toolbar.setBorder( BorderFactory.createEmptyBorder( 2, 3, 0, 4 ));
			mainSplit.setLeftComponent( inspectorPanel );
			mainSplit.setResizeWeight( 0.2 );
			
			toolbar.setOrientation( JToolBar.VERTICAL );
			
			add(p, BorderLayout.WEST  );
		}

		add(mainSplit, BorderLayout.CENTER);
		
	}
	
	private JXToolBar createInspectButtons()
	{
		inspectToolbar = UISupport.createToolbar();
		if( orientation == JSplitPane.VERTICAL_SPLIT )
			inspectToolbar.addSpace( 10 );
		inspectToolbar.setBackground( Color.WHITE );
		inspectToolbar.setOpaque( true );
		return inspectToolbar;
	}
	
	public float getDefaultDividerLocation()
	{
		return defaultDividerLocation;
	}

	public void setDefaultDividerLocation( float defaultDividerLocation )
	{
		this.defaultDividerLocation = defaultDividerLocation;
	}

	public <T extends Inspector> T addInspector( final T inspector )
	{
		if( inspectors.size() > 0 )
		{
			inspectToolbar.addSeparator();
		}
		
		inspectors.add( inspector );
		inspector.addPropertyChangeListener( JInspectorPanel.this );
		
		inspectorPanel.add( inspector.getComponent(), inspector.getInspectorId() );
		JToggleButton button = new JToggleButton( new SelectInspectorAction( inspector ));
		
		inspectorButtons.put( inspector, button );
		if( orientation == JSplitPane.HORIZONTAL_SPLIT )
		{
			String text = button.getText();
			button.setText( null );
			button.setPreferredSize( new Dimension( 17, 10 ) );
			button.setIcon( new VTextIcon( inspectToolbar, text, VTextIcon.ROTATE_LEFT) );
			inspectToolbar.add( button );
		}
		else
			inspectToolbar.add( button );
		
		inspectToolbar.invalidate();
		repaint();

		return inspector;
	}
	
	public Inspector getInspector( String inspectorId  )
	{
		for( Inspector inspector : inspectors )
		{
			if( inspector.getInspectorId().equals( inspectorId ))
				return inspector;
		}
		
		return null;
	}
	
	public Inspector getInspectorByTitle( String title )
	{
		for( Inspector inspector : inspectors )
		{
			if( inspector.getTitle().equals( title ))
				return inspector;
		}
		
		return null;
	}
	
	public void propertyChange(PropertyChangeEvent evt)
	{
		if( evt.getPropertyName().equals( Inspector.ENABLED_PROPERTY ))
		{
			JToggleButton toggleButton = inspectorButtons.get( evt.getSource() );
			toggleButton.setEnabled( ( Boolean ) evt.getNewValue() );
		}
	}
	
	public class SelectInspectorAction extends AbstractAction implements PropertyChangeListener
	{
		private final Inspector inspector;

		public SelectInspectorAction( Inspector inspector )
		{
			super( inspector.getTitle());
			this.inspector = inspector;
			
			putValue( AbstractAction.SHORT_DESCRIPTION, inspector.getDescription() );
			putValue( AbstractAction.SMALL_ICON, inspector.getIcon());
			setEnabled( inspector.isEnabled() );
			
			inspector.addPropertyChangeListener( this );
		}

		public void actionPerformed( ActionEvent arg0 )
		{
			JToggleButton button = inspectorButtons.get( inspector );
			if( !button.isSelected() )
			{
				currentInspector = null;
				button.setBackground( inspectToolbar.getBackground() );
				lastDividerLocation = mainSplit.getDividerLocation();
				inspectorPanel.setVisible( false );
			}
			else
			{
				activate( inspector );
			}
		}

		public void propertyChange( PropertyChangeEvent evt )
		{
			if( evt.getPropertyName().equals( Inspector.TITLE_PROPERTY ))
				putValue( AbstractAction.NAME, evt.getNewValue() );
			else if( evt.getPropertyName().equals( Inspector.ICON_PROPERTY ))
				putValue( AbstractAction.SMALL_ICON, evt.getNewValue() );
			else if( evt.getPropertyName().equals( Inspector.DESCRIPTION_PROPERTY ))
				putValue( AbstractAction.SHORT_DESCRIPTION, evt.getNewValue() );
			else if( evt.getPropertyName().equals( Inspector.ENABLED_PROPERTY ))
			{
				boolean enable = ((Boolean)evt.getNewValue()).booleanValue();
				setEnabled( enable );
				
				if( !enable && currentInspector == inspector )
				{
					inspectorButtons.get( currentInspector ).setSelected( false );
				}
			}
		}
	}
	
	public void release()
	{
		for( Inspector inspector : inspectors )
		{
			inspector.removePropertyChangeListener( this );
			inspector.release();
		}
		
		inspectors.clear();
		inspectorPanel.removeAll();
		mainSplit.removeAll();
	}

	public List<Inspector> getInspectors()
	{
		return inspectors;
	}

	public Inspector getCurrentInspector()
	{
		return currentInspector;
	}

	public void setInspectorVisible( boolean b )
	{
		inspectorPanel.setVisible( b );
	}

	public void setToolbarVisible( boolean b )
	{
		inspectToolbar.setVisible( b );
	}

	public double getResizeWeight()
	{
		return mainSplit.getResizeWeight();
	}

	public void setResizeWeight( double value )
	{
		mainSplit.setResizeWeight( value );
	}

	public int getDividerLocation()
	{
		return mainSplit.getDividerLocation();
	}

	public void setResetDividerLocation()
	{
		mainSplit.setDividerLocation( defaultDividerLocation );
	}

	public void setDividerLocation( int dividerLocation )
	{
		mainSplit.setDividerLocation( dividerLocation );
	}

	public void setCurrentInspector( String string )
	{
		for( Inspector inspector : inspectors )
		{
			if( inspector.getTitle().equals( string ))
			{
				activate( inspector );
				break;
			}
		}
	}
	
	public void deactivate()
	{
		activate( null );
	}
	
	public void activate( Inspector inspector )
	{
		if( inspector == currentInspector )
			return;
		
		if( currentInspector != null )
		{
			inspectorButtons.get( currentInspector ).setSelected( false );
			currentInspector.deactivate();
		}
		
		if( inspector == null )
		{
			currentInspector = null;
			inspectorPanel.setVisible( false );
		}
		else
		{
			JToggleButton button = inspectorButtons.get( inspector );
			currentInspector = inspector;
			
			button.setSelected( true );
			button.setBackground( Color.WHITE );
			
			if( !inspectorPanel.isVisible() )
			{
				inspectorPanel.setVisible( true );
				if( lastDividerLocation == 0 )
					mainSplit.setDividerLocation( defaultDividerLocation );
				else
					mainSplit.setDividerLocation( lastDividerLocation );
			}
			
			CardLayout cards = ( CardLayout ) inspectorPanel.getLayout();
			cards.show( inspectorPanel, inspector.getInspectorId() );
			
			currentInspector.activate();
		}
	}

	public void setContentComponent( JComponent content )
	{
		mainSplit.setTopComponent( content );
	}
}
