/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support.components;

import java.awt.Font;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.text.JTextComponent;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * Utility-class for creating forms
 */

public class SimpleForm
{
   private JPanel panel;
   private CellConstraints cc = new CellConstraints();
   private FormLayout layout;
   private RowSpec rowSpec;
   private int rowSpacing = 5;
   private Map<String,JComponent> components = new HashMap<String,JComponent>();
   private Map<JComboBox,Object[]> comboBoxMaps = new HashMap<JComboBox,Object[]>();
   private String rowAlignment = "top";
   private Map<String,String> hiddenValues;
	private boolean appended;
	private Font labelFont;

   public SimpleForm()
   {
      this( 5 );
   }
   
   public SimpleForm( String layout )
   {
      this( new FormLayout( layout ) );
   }

   public SimpleForm( FormLayout layout )
   {
      this.layout = layout;
      panel = new JPanel( layout );
      rowSpec = new RowSpec( rowAlignment + ":pref" );
   }

   public SimpleForm( int indent )
	{
   	 this( indent + "px:none,right:pref,10px,left:default,5px:grow(1.0)" );
	}

	public JPanel getPanel()
   {
      return panel;
   }

   public String getRowAlignment()
   {
      return rowAlignment;
   }

   public Font getLabelFont()
	{
		return labelFont;
	}

	public void setLabelFont( Font labelFont )
	{
		this.labelFont = labelFont;
	}

	public void setRowAlignment( String rowAlignment )
   {
      this.rowAlignment = rowAlignment;
      rowSpec = new RowSpec( rowAlignment + ":pref" );
   }

   public int getRowSpacing()
   {
      return rowSpacing;
   }

   public void setRowSpacing( int rowSpacing )
   {
      this.rowSpacing = rowSpacing;
   }

   public void addHiddenValue( String name, String value )
   {
      if( hiddenValues == null )
         hiddenValues = new HashMap<String,String>();

      hiddenValues.put( name, value );
   }

   public JButton addRightButton( Action action )
   {
      if( rowSpacing > 0 && !components.isEmpty() )
         addSpace( rowSpacing );

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      JButton button = new JButton( action );
      panel.add( button, cc.xy( 4, row, "right,bottom" ) );
      return button;
   }

   public void addSpace()
   {
   	addSpace( rowSpacing );
   }
   
   public void addSpace( int size )
   {
      if( size > 0 )
         layout.appendRow( new RowSpec( size + "px" ) );
   }

   public void addRightComponent( JComponent component )
   {
      if( rowSpacing > 0 && !components.isEmpty() )
         addSpace( rowSpacing );

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      panel.add( component, cc.xy( 4, row, "right,bottom" ) );
   }

   public JCheckBox appendCheckBox( String caption, String label, boolean selected )
   {
      JCheckBox checkBox = new JCheckBox( label, selected );
      components.put( caption, checkBox );
      append( caption, checkBox );
      return checkBox;
   }

   public void append( String label, JComponent component )
   {
      append( label, component, null );
   }

   public JComboBox appendComboBox( String label, Map values )
   {
      Object[] valueArray = new Object[values.size()];
      Object[] keyArray = new Object[values.size()];

      int ix = 0;
      for( Iterator i = values.keySet().iterator(); i.hasNext(); ix++ )
      {
         keyArray[ix] = i.next();
         valueArray[ix] = values.get( keyArray[ix] );
      }

      JComboBox comboBox = new JComboBox( valueArray );

      comboBoxMaps.put( comboBox, keyArray );

      append( label, comboBox );
      return comboBox;
   }

   public JComboBox appendComboBox( String label, Object [] values, String tooltip )
   {
      JComboBox comboBox = new JComboBox( values );
      comboBox.setToolTipText( tooltip );
      append( label, comboBox );
      return comboBox;
   }
   
   public JComboBox appendComboBox( String label, ComboBoxModel model, String tooltip )
   {
      JComboBox comboBox = new JComboBox( model );
      comboBox.setToolTipText( tooltip );
      append( label, comboBox );
      return comboBox;
   }

   public void appendFixed( String label, JComponent component )
   {
      append( label, component, "left:pref" );
   }

   public void append( String label, JComponent component, String alignments )
   {
   	int spaceRowIndex = -1;
   	
      if( rowSpacing > 0 && appended )
      {
         addSpace( rowSpacing );
         spaceRowIndex = layout.getRowCount();
      }

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      if( label != null )
      {
         JLabel jlabel = new JLabel( label );
         jlabel.setBorder( BorderFactory.createEmptyBorder( 3, 0, 0, 0 ) );
         if( labelFont != null )
         	jlabel.setFont( labelFont );
         
			panel.add( jlabel, cc.xy( 2, row ) );
			
			component.addComponentListener( new LabelHider( jlabel, spaceRowIndex ) );
      }
      else
      	component.addComponentListener( new LabelHider( null, spaceRowIndex ) );

      if( alignments == null )
         panel.add( component, cc.xy( 4, row ) );
      else
         panel.add( component, cc.xy( 4, row, alignments ) );

      components.put( label, component );
      appended = true;
   }

   public void appendSeparator()
   {
   	if( appended && rowSpacing > 0 )
   		addSpace( rowSpacing );

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      panel.add( new JSeparator(), cc.xywh( 2, row, 3, 1 ) );
      appended = true;
   }

   public JTextField appendTextField( String label, String tooltip )
   {
      JTextField textField = new JTextField();
      textField.setColumns( 30 );
      textField.setToolTipText( tooltip );
		append( label, textField );
		return textField;
   }
   
   public JTextField appendPasswordField( String label, String tooltip )
   {
      JPasswordField textField = new JPasswordField();
      textField.setColumns( 30 );
      textField.setToolTipText( tooltip );
		append( label, textField );
		return textField;
   }

   public void setComponentValue( String label, String value )
   {
      JComponent component = getComponent( label );

      if( component instanceof JTextComponent )
      {
         ((JTextComponent) component).setText( value );
      }
      else if( component instanceof JComboBox )
      {
         JComboBox comboBox = ((JComboBox) component);
         comboBox.setSelectedItem( value );
      }
      else if( component instanceof JList )
      {
         ((JList) component).setSelectedValue( value, true );
      }
      else if( component instanceof JCheckBox )
      {
         ((JCheckBox) component).setSelected( Boolean.valueOf( value ));
      }
      else if( component instanceof JFormComponent )
      {
      	((JFormComponent) component).setValue( value );
      }      	
   }
   
   public String getComponentValue( String label )
   {
      JComponent component = getComponent( label );
      if( component == null )
      {
         return (String) (hiddenValues == null ? null : hiddenValues.get( label ));
      }
      
      if( component instanceof JTextComponent )
      {
         return ((JTextComponent) component).getText();
      }

      if( component instanceof JComboBox )
      {
         JComboBox comboBox = ((JComboBox) component);
         int selectedIndex = comboBox.getSelectedIndex();
         if( selectedIndex != -1 )
         {
	         if( comboBoxMaps.containsKey( component ) )
	         {
	            Object[] keys = (Object[]) comboBoxMaps.get( comboBox );
	            Object value = keys[selectedIndex];
					return (String) value == null ? null : value.toString(); // Added support for enums
	         }
	         else
	         {
	            Object value = comboBox.getSelectedItem();
	            return (String) value == null ? null : value.toString(); // Added support for enums
	         }
         }
      }

      if( component instanceof JList )
      {
         return (String) ((JList) component).getSelectedValue();
      }

      if( component instanceof JCheckBox )
      {
         return String.valueOf( ((JCheckBox) component).isSelected() );
      }

      else if( component instanceof JFormComponent )
      {
      	return ((JFormComponent) component).getValue();
      }      	
      
      return null;
   }

   public JComponent getComponent( String label )
   {
      return (JComponent) components.get( label );
   }

	public void setBorder(Border border)
	{
		panel.setBorder( border );
	}

	public int getRowCount()
	{
		return layout.getRowCount();
	}
	
	public void addComponent(JComponent component)
	{
		layout.appendRow( rowSpec );
      int row = layout.getRowCount();
		
		panel.add( component, cc.xyw( 2, row, 4 ) );
	}

	public void setValues(Map<String, String> values)
	{
		for( Iterator<String> i = values.keySet().iterator(); i.hasNext(); )
		{
			String key = i.next();
			setComponentValue( key, values.get( key ));
		}
	}

	public void getValues(Map<String, String> values)
	{
		for( Iterator<String> i = components.keySet().iterator(); i.hasNext(); )
		{
			String key = i.next();
			values.put( key, getComponentValue( key ));
		}
	}

	public void append( JComponent component )
	{
		int spaceRowIndex = -1;
   	
      if( rowSpacing > 0 && appended )
      {
         addSpace( rowSpacing );
         spaceRowIndex = layout.getRowCount();
      }

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      panel.add( component, cc.xyw( 2, row, 4 ) );
      
      component.addComponentListener( new LabelHider( null, spaceRowIndex ) );
      
      appended = true;
	}
	
	private final class LabelHider extends ComponentAdapter
	{
		private final JLabel jlabel;
		private final int rowIndex;

		public LabelHider( JLabel jlabel, int i )
		{
			this.jlabel = jlabel;
			this.rowIndex = i;
		}

		public void componentHidden( ComponentEvent e )
		{
			if( jlabel != null )
				jlabel.setVisible( false );
			
			if( rowIndex >= 0 && rowIndex < layout.getRowCount() )
				layout.setRowSpec( rowIndex, new RowSpec( "0px" ));
		}

		public void componentShown( ComponentEvent e )
		{
			if( jlabel != null )
				jlabel.setVisible( true );
			
			if( rowIndex >= 0 && rowIndex < layout.getRowCount() )
				layout.setRowSpec( rowIndex, new RowSpec( rowSpacing + "px" ));
		}
	}
}

